class CleanSlate
  instance_methods.each { |m| undef_method m unless m.to_s =~ /^__/ }
end

class Paged < CleanSlate
  def initialize(items, opts)
    raise ArgumentError.new ':page_size must be a positive number' unless opts[:page_size] and opts[:page_size] > 0
    @items = items
    @page_size = opts[:page_size]
    @page = 0
  end
  
  def page=(page)
    raise ArgumentError.new 'page must be a number' if not page.kind_of? Numeric
    @page = page - 1
  end
  
  def page
    @page + 1
  end
  
  def previous_page
    return if outside_bounds
    page - 1 if page > 1
  end
  
  def next_page
    return if outside_bounds
    page + 1 if page < total_pages
  end
  
  def respond_to?(symbol, include_private=false)
    @items.respond_to? symbol, include_private
  end
  
private

  def method_missing(symbol, *args, &block)
    page_of_items.send(symbol, *args, &block)
  end
  
  def page_of_items
    return [] if outside_bounds
    @items[start_index, @page_size]
  end

  def start_index
    @page * @page_size
  end
  
  def total_pages
    (@items.size.quo @page_size).ceil
  end
  
  def outside_bounds
    @page < 0 or @page > total_pages
  end
end